@@ -1,6 +1,6 @@
-import functools
+import logging
 
-from ..util import properties
+from .._compat import properties
 from ..backend import KeyringBackend
 from ..credentials import SimpleCredential
 from ..errors import PasswordDeleteError, ExceptionRaisedContext
@@ -22,8 +22,7 @@ with ExceptionRaisedContext() as missing_deps:
         # force demand import to raise ImportError
         win32cred.__name__
 
-
-__metaclass__ = type
+log = logging.getLogger(__name__)
 
 
 class Persistence:
@@ -43,6 +42,24 @@ class Persistence:
         setattr(keyring, '_persist', value)
 
 
+class DecodingCredential(dict):
+    @property
+    def value(self):
+        """
+        Attempt to decode the credential blob as UTF-16 then UTF-8.
+        """
+        cred = self['CredentialBlob']
+        try:
+            return cred.decode('utf-16')
+        except UnicodeDecodeError:
+            decoded_cred_utf8 = cred.decode('utf-8')
+            log.warning(
+                "Retrieved an UTF-8 encoded credential. Please be aware that "
+                "this library only writes credentials in UTF-16."
+            )
+            return decoded_cred_utf8
+
+
 class WinVaultKeyring(KeyringBackend):
     """
     WinVaultKeyring stores encrypted passwords using the Windows Credential
@@ -52,7 +69,7 @@ class WinVaultKeyring(KeyringBackend):
 
     This backend does some gymnastics to simulate multi-user support,
     which WinVault doesn't support natively. See
-    https://bitbucket.org/kang/python-keyring-lib/issue/47/winvaultkeyring-only-ever-returns-last#comment-731977
+    https://github.com/jaraco/keyring/issues/47#issuecomment-75763152
     for details on the implementation, but here's the gist:
 
     Passwords are stored under the service name unless there is a collision
@@ -63,9 +80,8 @@ class WinVaultKeyring(KeyringBackend):
 
     persist = Persistence()
 
-    @properties.ClassProperty
-    @classmethod
-    def priority(cls):
+    @properties.classproperty
+    def priority(cls) -> int:
         """
         If available, the preferred backend on Windows.
         """
@@ -75,7 +91,7 @@ class WinVaultKeyring(KeyringBackend):
 
     @staticmethod
     def _compound_name(username, service):
-        return '%(username)s@%(service)s' % vars()
+        return f'{username}@{service}'
 
     def get_password(self, service, username):
         # first attempt to get the password under the service name
@@ -85,8 +101,7 @@ class WinVaultKeyring(KeyringBackend):
             res = self._get_password(self._compound_name(username, service))
         if not res:
             return None
-        blob = res['CredentialBlob']
-        return blob.decode('utf-16')
+        return res.value
 
     def _get_password(self, target):
         try:
@@ -94,11 +109,10 @@ class WinVaultKeyring(KeyringBackend):
                 Type=win32cred.CRED_TYPE_GENERIC, TargetName=target
             )
         except pywintypes.error as e:
-            e = OldPywinError.wrap(e)
             if e.winerror == 1168 and e.funcname == 'CredRead':  # not found
                 return None
             raise
-        return res
+        return DecodingCredential(res)
 
     def set_password(self, service, username, password):
         existing_pw = self._get_password(service)
@@ -109,7 +123,7 @@ class WinVaultKeyring(KeyringBackend):
             self._set_password(
                 target,
                 existing_username,
-                existing_pw['CredentialBlob'].decode('utf-16'),
+                existing_pw.value,
             )
         self._set_password(service, username, str(password))
 
@@ -139,7 +153,6 @@ class WinVaultKeyring(KeyringBackend):
         try:
             win32cred.CredDelete(Type=win32cred.CRED_TYPE_GENERIC, TargetName=target)
         except pywintypes.error as e:
-            e = OldPywinError.wrap(e)
             if e.winerror == 1168 and e.funcname == 'CredDelete':  # not found
                 return
             raise
@@ -154,28 +167,4 @@ class WinVaultKeyring(KeyringBackend):
             res = self._get_password(service)
             if not res:
                 return None
-        return SimpleCredential(res['UserName'], res['CredentialBlob'].decode('utf-16'))
-
-
-class OldPywinError:
-    """
-    A compatibility wrapper for old PyWin32 errors, such as reported in
-    https://bitbucket.org/kang/python-keyring-lib/issue/140/
-    """
-
-    def __init__(self, orig):
-        self.orig = orig
-
-    @property
-    def funcname(self):
-        return self.orig[1]
-
-    @property
-    def winerror(self):
-        return self.orig[0]
-
-    @classmethod
-    def wrap(cls, orig_err):
-        attr_check = functools.partial(hasattr, orig_err)
-        is_old = not all(map(attr_check, ['funcname', 'winerror']))
-        return cls(orig_err) if is_old else orig_err
+        return SimpleCredential(res['UserName'], res.value)
